# from .report import *
from rephorm.objects.report import Report
from rephorm.objects.table_section import TableSection
from rephorm.objects.table import Table
from rephorm.objects.page_break import PageBreak
from rephorm.objects.grid import Grid
from rephorm.objects.chart import Chart
from rephorm.objects.chapter import Chapter
from .objects.text import Text
from rephorm.objects.footnote import Footnote
from rephorm.objects.chart_series import ChartSeries
from rephorm.objects.table_series import TableSeries

__version__ = "1.0.2"
__author__ = 'OGResearch team'
__credits__ = 'OGResearch'
__license__ = 'Copyright 2025 OGResearch, all rights reserved'


